@@ -2,9 +2,9 @@
 
 #include "Python.h"
 
-static char cprt[] =
+static const char cprt[] =
 "\
-Copyright (c) 2001-2012 Python Software Foundation.\n\
+Copyright (c) 2001-2017 Python Software Foundation.\n\
 All Rights Reserved.\n\
 \n\
 Copyright (c) 2000 BeOpen.com.\n\
